@@ -1,13 +1,13 @@
 """Pickle field implementation for Django."""
+from base64 import b64decode, b64encode
 from copy import deepcopy
-from base64 import b64encode, b64decode
 from zlib import compress, decompress
-import six
-import django
-from django.db import models
 
-from picklefield import DEFAULT_PROTOCOL
-from picklefield.compat import force_text, loads, dumps
+from django.db import models
+from django.utils.encoding import force_text
+
+from . import DEFAULT_PROTOCOL
+from .compat import dumps, loads
 
 
 class PickledObject(str):
@@ -26,10 +26,10 @@ class PickledObject(str):
 
 class _ObjectWrapper(object):
     """
-    A class used to wrap object that have properties that may clash with the 
+    A class used to wrap object that have properties that may clash with the
     ORM internals.
-    
-    For example, objects with the `prepare_database_save` property such as 
+
+    For example, objects with the `prepare_database_save` property such as
     `django.db.Model` subclasses won't work under certain conditions and the
     same apply for trying to retrieve any `callable` object.
     """
@@ -45,38 +45,33 @@ def wrap_conflictual_object(obj):
     return obj
 
 
-def dbsafe_encode(value, compress_object=False, pickle_protocol=DEFAULT_PROTOCOL):
+def dbsafe_encode(value, compress_object=False, pickle_protocol=DEFAULT_PROTOCOL, copy=True):
     # We use deepcopy() here to avoid a problem with cPickle, where dumps
     # can generate different character streams for same lookup value if
     # they are referenced differently.
     # The reason this is important is because we do all of our lookups as
     # simple string matches, thus the character streams must be the same
     # for the lookups to work properly. See tests.py for more information.
-    value = dumps(deepcopy(value), protocol=pickle_protocol)
+    if copy:
+        # Copy can be very expensive if users aren't going to perform lookups
+        # on the value anyway.
+        value = deepcopy(value)
+    value = dumps(value, protocol=pickle_protocol)
     if compress_object:
         value = compress(value)
-    value = b64encode(value).decode() # decode bytes to str
+    value = b64encode(value).decode()  # decode bytes to str
     return PickledObject(value)
 
 
 def dbsafe_decode(value, compress_object=False):
-    value = value.encode() # encode str to bytes
+    value = value.encode()  # encode str to bytes
     value = b64decode(value)
     if compress_object:
         value = decompress(value)
     return loads(value)
 
 
-def _get_subfield_superclass():
-    # hardcore trick to support django < 1.3 - there was something wrong with
-    # inheritance and SubfieldBase before django 1.3
-    # see https://github.com/django/django/commit/222c73261650201f5ce99e8dd4b1ce0d30a69eb4
-    if django.VERSION < (1,3):
-        return models.Field
-    return six.with_metaclass(models.SubfieldBase, models.Field)
-
-
-class PickledObjectField(_get_subfield_superclass()):
+class PickledObjectField(models.Field):
     """
     A field that will accept *any* python object and store it in the
     database. PickledObjectField will optionally compress its values if
@@ -86,11 +81,11 @@ class PickledObjectField(_get_subfield_superclass()):
     can still do lookups using None). This way, it is still possible to
     use the ``isnull`` lookup type correctly.
     """
-    __metaclass__ = models.SubfieldBase  # for django < 1.3
 
     def __init__(self, *args, **kwargs):
         self.compress = kwargs.pop('compress', False)
         self.protocol = kwargs.pop('protocol', DEFAULT_PROTOCOL)
+        self.copy = kwargs.pop('copy', True)
         kwargs.setdefault('editable', False)
         super(PickledObjectField, self).__init__(*args, **kwargs)
 
@@ -140,6 +135,9 @@ class PickledObjectField(_get_subfield_superclass()):
         value = super(PickledObjectField, self).pre_save(model_instance, add)
         return wrap_conflictual_object(value)
 
+    def from_db_value(self, value, expression, connection, context):
+        return self.to_python(value)
+
     def get_db_prep_value(self, value, connection=None, prepared=False):
         """
         Pickle and b64encode the object, optionally compressing it.
@@ -158,35 +156,20 @@ class PickledObjectField(_get_subfield_superclass()):
             # marshaller (telling it to store it like it would a string), but
             # since both of these methods result in the same value being stored,
             # doing things this way is much easier.
-            value = force_text(dbsafe_encode(value, self.compress, self.protocol))
+            value = force_text(dbsafe_encode(value, self.compress, self.protocol, self.copy))
         return value
 
     def value_to_string(self, obj):
-        value = self._get_val_from_obj(obj)
+        value = self.value_from_object(obj)
         return self.get_db_prep_value(value)
 
     def get_internal_type(self):
         return 'TextField'
 
-    def get_db_prep_lookup(self, lookup_type, value, connection=None, prepared=False):
-        if lookup_type not in ['exact', 'in', 'isnull']:
-            raise TypeError('Lookup type %s is not supported.' % lookup_type)
-        # The Field model already calls get_db_prep_value before doing the
-        # actual lookup, so all we need to do is limit the lookup types.
-        try:
-            return super(PickledObjectField, self).get_db_prep_lookup(
-                lookup_type, value, connection=connection, prepared=prepared)
-        except TypeError:
-            # Try not to break on older versions of Django, where the
-            # `connection` and `prepared` parameters are not available.
-            return super(PickledObjectField, self).get_db_prep_lookup(
-                lookup_type, value)
-
-
-# South support; see http://south.aeracode.org/docs/tutorial/part4.html#simple-inheritance
-try:
-    from south.modelsinspector import add_introspection_rules
-except ImportError:
-    pass
-else:
-    add_introspection_rules([], [r"^picklefield\.fields\.PickledObjectField"])
+    def get_lookup(self, lookup_name):
+        """
+        We need to limit the lookup types.
+        """
+        if lookup_name not in ['exact', 'in', 'isnull']:
+            raise TypeError('Lookup type %s is not supported.' % lookup_name)
+        return super(PickledObjectField, self).get_lookup(lookup_name)
